@@ -16,6 +16,7 @@ import urllib
 from urlparse import urlparse
 
 import mozinfo
+import mozlog
 
 from parser import DirectoryParser
 from timezones import PacificTimezone
@@ -26,7 +27,7 @@ import progressbar
 version = pkg_resources.require("mozdownload")[0].version
 
 __doc__ = """
-Module to handle downloads for different types of ftp.mozilla.org hosted
+Module to handle downloads for different types of ftp.mozilla.org hosted \
 applications.
 
 mozdownload version: %(version)s
@@ -81,7 +82,8 @@ class Scraper(object):
     def __init__(self, directory, version, platform=None,
                  application='firefox', locale='en-US', extension=None,
                  authentication=None, retry_attempts=0, retry_delay=10.,
-                 is_stub_installer=False, timeout=None):
+                 is_stub_installer=False, timeout=None, log_level='INFO',
+                 base_url=BASE_URL):
 
         # Private properties for caching
         self._target = None
@@ -99,9 +101,12 @@ class Scraper(object):
         self.timeout_download = timeout
         self.timeout_network = 60.
 
+        self.logger = mozlog.getLogger(' ')
+        self.logger.setLevel(getattr(mozlog, log_level.upper()))
+
         # build the base URL
         self.application = application
-        self.base_url = urljoin(BASE_URL, self.application)
+        self.base_url = urljoin(base_url, self.application)
 
         attempt = 0
         while True:
@@ -111,11 +116,12 @@ class Scraper(object):
                 break
             except (NotFoundError, requests.exceptions.RequestException), e:
                 if self.retry_attempts > 0:
-                    # Print only if multiple attempts are requested
-                    print "Build not found: '%s'" % e.message
-                    print 'Will retry in %s seconds...' % self.retry_delay
+                    # Log only if multiple attempts are requested
+                    self.logger.warning("Build not found: '%s'" % e.message)
+                    self.logger.info('Will retry in %s seconds...' %
+                                     (self.retry_delay))
                     time.sleep(self.retry_delay)
-                    print "Retrying... (attempt %s)" % attempt
+                    self.logger.info("Retrying... (attempt %s)" % attempt)
 
                 if attempt >= self.retry_attempts:
                     if hasattr(e, 'response') and \
@@ -155,11 +161,12 @@ class Scraper(object):
                                         self.path)
             except (NotFoundError, requests.exceptions.RequestException), e:
                 if self.retry_attempts > 0:
-                    # Print only if multiple attempts are requested
-                    print "Build not found: '%s'" % e.message
-                    print 'Will retry in %s seconds...' % self.retry_delay
+                    # Log only if multiple attempts are requested
+                    self.logger.warning("Build not found: '%s'" % e.message)
+                    self.logger.info('Will retry in %s seconds...' %
+                                     (self.retry_delay))
                     time.sleep(self.retry_delay)
-                    print "Retrying... (attempt %s)" % attempt
+                    self.logger.info("Retrying... (attempt %s)" % attempt)
 
                 if attempt >= self.retry_attempts:
                     if hasattr(e, 'response') and \
@@ -248,10 +255,12 @@ class Scraper(object):
 
         # Don't re-download the file
         if os.path.isfile(os.path.abspath(self.target)):
-            print "File has already been downloaded: %s" % (self.target)
+            self.logger.info("File has already been downloaded: %s" %
+                             (self.target))
             return
 
-        print 'Downloading from: %s' % (urllib.unquote(self.final_url))
+        self.logger.info('Downloading from: %s' %
+                         (urllib.unquote(self.final_url)))
         tmp_file = self.target + ".part"
 
         while True:
@@ -290,11 +299,12 @@ class Scraper(object):
                 if tmp_file and os.path.isfile(tmp_file):
                     os.remove(tmp_file)
                 if self.retry_attempts > 0:
-                    # Print only if multiple attempts are requested
-                    print 'Download failed: "%s"' % e.message
-                    print 'Will retry in %s seconds...' % self.retry_delay
+                    # Log only if multiple attempts are requested
+                    self.logger.warning('Download failed: "%s"' % e.message)
+                    self.logger.info('Will retry in %s seconds...' %
+                                     (self.retry_delay))
                     time.sleep(self.retry_delay)
-                    print "Retrying... (attempt %s)" % attempt
+                    self.logger.info("Retrying... (attempt %s)" % attempt)
                 if attempt >= self.retry_attempts:
                     raise
                 time.sleep(self.retry_delay)
@@ -303,12 +313,12 @@ class Scraper(object):
 
     def show_matching_builds(self, builds):
         """Output the matching builds"""
-        print 'Found %s build%s: %s' % (
+        self.logger.info('Found %s build%s: %s' % (
             len(builds),
             len(builds) > 1 and 's' or '',
             len(builds) > 10 and
             ' ... '.join([', '.join(builds[:5]), ', '.join(builds[-5:])]) or
-            ', '.join(builds))
+            ', '.join(builds)))
 
 
 class DailyScraper(Scraper):
@@ -338,42 +348,42 @@ class DailyScraper(Scraper):
             # date and time can be extracted:
             # '20111212042025' -> '2011-12-12 04:20:25'
             self.date = datetime.strptime(self.build_id, '%Y%m%d%H%M%S')
-            self.builds, self.build_index = self.get_build_info_for_date(
-                self.date, has_time=True)
 
         elif self.date:
             # A date (without time) has been specified. Use its value and the
             # build index to find the requested build for that day.
             self.date = datetime.strptime(self.date, '%Y-%m-%d')
-            self.builds, self.build_index = self.get_build_info_for_date(
-                self.date, build_index=self.build_index)
 
         else:
-            # If no build id nor date have been specified the lastest available
+            # If no build id nor date have been specified the latest available
             # build of the given branch has to be identified. We also have to
             # retrieve the date of the build via its build id.
-            url = '%s/nightly/latest-%s/' % (self.base_url, self.branch)
+            self.date = self.get_latest_build_date()
 
-            print 'Retrieving the build status file from %s' % url
-            parser = DirectoryParser(url, authentication=self.authentication,
-                                     timeout=self.timeout_network)
-            parser.entries = parser.filter(r'.*%s\.txt' % self.platform_regex)
-            if not parser.entries:
-                message = 'Status file for %s build cannot be found' % \
-                    self.platform_regex
-                raise NotFoundError(message, url)
+        self.builds, self.build_index = self.get_build_info_for_date(
+            self.date, self.build_index)
 
-            # Read status file for the platform, retrieve build id,
-            # and convert to a date
-            headers = {'Cache-Control': 'max-age=0'}
-            r = requests.get(url + parser.entries[-1],
-                             auth=self.authentication, headers=headers)
-            r.raise_for_status()
+    def get_latest_build_date(self):
+        """ Returns date of latest available nightly build."""
+        url = urljoin(self.base_url, 'nightly', 'latest-%s/' % self.branch)
 
-            self.date = datetime.strptime(r.text.split('\n')[0],
-                                          '%Y%m%d%H%M%S')
-            self.builds, self.build_index = self.get_build_info_for_date(
-                self.date, has_time=True)
+        self.logger.info('Retrieving the build status file from %s' % url)
+        parser = DirectoryParser(url, authentication=self.authentication,
+                                 timeout=self.timeout_network)
+        parser.entries = parser.filter(r'.*%s\.txt' % self.platform_regex)
+        if not parser.entries:
+            message = 'Status file for %s build cannot be found' % \
+                self.platform_regex
+            raise NotFoundError(message, url)
+
+        # Read status file for the platform, retrieve build id,
+        # and convert to a date
+        headers = {'Cache-Control': 'max-age=0'}
+        r = requests.get(url + parser.entries[-1],
+                         auth=self.authentication, headers=headers)
+        r.raise_for_status()
+
+        return datetime.strptime(r.text.split('\n')[0], '%Y%m%d%H%M%S')
 
     def is_build_dir(self, dir):
         """Return whether or not the given dir contains a build."""
@@ -392,10 +402,11 @@ class DailyScraper(Scraper):
                 continue
         return False
 
-    def get_build_info_for_date(self, date, has_time=False, build_index=None):
+    def get_build_info_for_date(self, date, build_index=None):
         url = urljoin(self.base_url, self.monthly_build_list_regex)
+        has_time = date and date.time()
 
-        print 'Retrieving list of builds from %s' % url
+        self.logger.info('Retrieving list of builds from %s' % url)
         parser = DirectoryParser(url, authentication=self.authentication,
                                  timeout=self.timeout_network)
         regex = r'%(DATE)s-(\d+-)+%(BRANCH)s%(L10N)s$' % {
@@ -433,8 +444,8 @@ class DailyScraper(Scraper):
                         'linux64': r'\.%(EXT)s$',
                         'mac': r'\.%(EXT)s$',
                         'mac64': r'\.%(EXT)s$',
-                        'win32': r'(\.installer)%(STUB)s\.%(EXT)s$',
-                        'win64': r'(\.installer)%(STUB)s\.%(EXT)s$'}
+                        'win32': r'(\.installer%(STUB)s)?\.%(EXT)s$',
+                        'win64': r'(\.installer%(STUB)s)?\.%(EXT)s$'}
         regex = regex_base_name + regex_suffix[self.platform]
 
         return regex % {'APP': self.application,
@@ -557,17 +568,16 @@ class ReleaseCandidateScraper(ReleaseScraper):
         "Defines additional build information"
 
         # Internally we access builds via index
+        self.builds, self.build_index = self.get_build_info_for_version(
+            self.version)
         if self.build_number is not None:
             self.builds = ['build%s' % self.build_number]
             self.build_index = 0
-        else:
-            self.builds, self.build_index = self.get_build_info_for_version(
-                self.version)
 
     def get_build_info_for_version(self, version, build_index=None):
         url = urljoin(self.base_url, self.candidate_build_list_regex)
 
-        print 'Retrieving list of candidate builds from %s' % url
+        self.logger.info('Retrieving list of candidate builds from %s' % url)
         parser = DirectoryParser(url, authentication=self.authentication,
                                  timeout=self.timeout_network)
         if not parser.entries:
@@ -623,15 +633,15 @@ class ReleaseCandidateScraper(ReleaseScraper):
             # Try to download the signed candidate build
             Scraper.download(self)
         except NotFoundError, e:
-            print str(e)
+            self.logger.exception(str(e))
 
             # If the signed build cannot be downloaded and unsigned builds are
             # allowed, try to download the unsigned build instead
             if self.no_unsigned:
                 raise
             else:
-                print "Signed build has not been found. Falling back to" \
-                      " unsigned build."
+                self.logger.warning("Signed build has not been found. "
+                                    "Falling back to unsigned build.")
                 self.unsigned = True
                 Scraper.download(self)
 
@@ -692,14 +702,15 @@ class TinderboxScraper(Scraper):
                         'linux64': r'.*\.%(EXT)s$',
                         'mac': r'.*\.%(EXT)s$',
                         'mac64': r'.*\.%(EXT)s$',
-                        'win32': r'.*(\.installer)\.%(EXT)s$',
-                        'win64': r'.*(\.installer)\.%(EXT)s$'}
+                        'win32': r'.*(\.installer%(STUB)s)\.%(EXT)s$',
+                        'win64': r'.*(\.installer%(STUB)s)\.%(EXT)s$'}
 
         regex = regex_base_name + regex_suffix[self.platform]
 
         return regex % {'APP': self.application,
                         'LOCALE': self.locale,
                         'PLATFORM': PLATFORM_FRAGMENTS[self.platform],
+                        'STUB': '-stub' if self.is_stub_installer else '',
                         'EXT': self.extension}
 
     def build_filename(self, binary):
@@ -753,7 +764,7 @@ class TinderboxScraper(Scraper):
     def get_build_info_for_index(self, build_index=None):
         url = urljoin(self.base_url, self.build_list_regex)
 
-        print 'Retrieving list of builds from %s' % url
+        self.logger.info('Retrieving list of builds from %s' % url)
         parser = DirectoryParser(url, authentication=self.authentication,
                                  timeout=self.timeout_network)
         parser.entries = parser.filter(r'^\d+$')
@@ -897,6 +908,12 @@ def cli():
                       metavar='TIMEOUT',
                       help='Amount of time (in seconds) until a download times'
                            ' out')
+    parser.add_option('--log-level',
+                      action='store',
+                      dest='log_level',
+                      default='INFO',
+                      metavar='LOG_LEVEL',
+                      help='Threshold for log output (default: %default)')
 
     # Option group for candidate builds
     group = OptionGroup(parser, "Candidate builds",
@@ -938,6 +955,14 @@ def cli():
     # TODO: option group for nightly builds
     (options, args) = parser.parse_args()
 
+    # Gives instructions to user when no arguments were passed
+    if len(sys.argv) == 1:
+        print __doc__
+        parser.print_usage()
+        print "Specify --help for more information on options. " \
+              "Please see the README for examples."
+        return
+
     # Check for required options and arguments
     # Note: Will be optional when ini file support has been landed
     if not options.url \
@@ -957,7 +982,9 @@ def cli():
                         'retry_attempts': options.retry_attempts,
                         'retry_delay': options.retry_delay,
                         'is_stub_installer': options.is_stub_installer,
-                        'timeout': options.timeout}
+                        'timeout': options.timeout,
+                        'log_level': options.log_level}
+
     scraper_options = {
         'candidate': {'build_number': options.build_number,
                       'no_unsigned': options.no_unsigned},
